#include <qapplication.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qdial.h>
#include <qiconview.h>
#include <qmap.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qlcdnumber.h>
#include <qframe.h>
#include <qgroupbox.h>

#include <qtimer.h>

#include <errno.h>

#include "../../src/qtxembed.h"

#include <unistd.h>
#include <stdlib.h>

class MyIconView : public QIconView
{
    Q_OBJECT
public:
    MyIconView::MyIconView(QWidget *parent = 0, const char *name = 0)
	: QIconView(parent, name)
    {
	connect(this, SIGNAL(dropped(QDropEvent*, const QValueList<QIconDragItem>&)),	
                SLOT(slotNewItem(QDropEvent*, const QValueList<QIconDragItem>&)));
    }

    QDragObject *MyIconView::dragObject()
    {
        return new QTextDrag(currentItem()->text(), this);
    }
    
private slots:
    void slotNewItem(QDropEvent *e, const QValueList<QIconDragItem> &)
    {
	if (!e->provides( "text/plain"))
	    return;

	QString tag;
	if (QTextDrag::decode(e, tag)) {
	    
	    QIconViewItem *item = new QIconViewItem(this, tag);
	    item->setRenameEnabled(true);
	    item->move(e->pos());
	}
	
	e->acceptAction();
    }

};

class DemoWidget : public QtXEmbedClient
{
    Q_OBJECT

public:
    DemoWidget::DemoWidget(const QString &argv0, int indent = 0, bool root = true, QWidget *parent = 0, const char *name = 0, WFlags = 0)
	: QtXEmbedClient(parent, name), arg0(argv0)
    {
	ind = indent;
	vlayout = new QVBoxLayout(this);

	addSibling();
    }

private slots:

    void addSibling()
    {
	QGroupBox *box = new QGroupBox("Level " + QString::number(ind), this);
	box->setInsideMargin(8);
	box->setColumnLayout(2, Horizontal);

	vlayout->addWidget(box);

	QHBox *hbox = new QHBox(box);
	QPushButton *button = new QPushButton("New Sibling", hbox, "Add Sibling Button in DemoWidget");
	QPushButton *childbutton = new QPushButton("New Child", hbox, "Add Child Button in DemoWidget");

	QPushButton *closebutton = new QPushButton("Close", box, "CloseButton in DemoWidget");
	
	QtXEmbedContainer *embed = new QtXEmbedContainer(box);
	connect(embed, SIGNAL(clientClosed()), SLOT(close()));
	containers[childbutton] = embed;
	
	connect(button, SIGNAL(clicked()), SLOT(addSibling()));
	connect(childbutton, SIGNAL(clicked()), SLOT(addChild()));
	connect(closebutton, SIGNAL(clicked()), SLOT(close()));

	qDebug("Container window = %li", embed->winId());
	
	box->show();
    }

    void addChild()
    {
	QtXEmbedContainer *embed = containers[(QObject *)sender()];
	if (fork() == 0) {
	    execl(arg0.latin1(), arg0.latin1(), QString::number(embed->winId()).latin1(),
		  QString::number(ind + 1).latin1(), NULL);
	    exit(1);
	}

	QPushButton *b = (QPushButton *)sender();
	b->setEnabled(false);
    }
    
private:
    QString arg0;
    int ind;
    QMap<QObject *, QtXEmbedContainer *> containers;
    QVBoxLayout *vlayout;
    
};

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    if (argc > 1) {
	Window id;
	if (argv[1][0] == '0') sscanf(argv[1], "0x%x", (unsigned int *)&id);
	else sscanf(argv[1], "%li", &id);

	DemoWidget demo(argv[0], argc > 2 ? atoi(argv[2]) : 0, false);
	
	app.setMainWidget(&demo);

	qDebug("Embedding client %li into %li", demo.winId(), id);
	demo.embedInto(id);
	demo.show();
	return app.exec();
    } else {
	DemoWidget demo(argv[0], 0, true);

	app.setMainWidget(&demo);

	demo.show();
	return app.exec();
    }
}

#include "main.moc"
